; -------------------------------------
; Z80 SRAM K1-bus micro computer board 
; with Interpreter Virtual Machine 
; (c) 2016 - 2017 kio@little-bat.de
; -------------------------------------


#target rom
#include "settings.s"


; -------------------------------------------
;		OPTIONS
; -------------------------------------------

; K1-bus:
;
option_a5	equ	1		; 1 => use A5 for rd=1/wr=0 selection, use A8 for K1-bus A5
					; used when use of WR results in spikes on the k1-bus strobe lines

bSIO_SELECT	equ 	1		; the SIO board is jumpered for D1
bIDE_SELECT	equ 	2		; the IDE board uses D2

mSIO_SELECT 	equ	0xff - 1<<bSIO_SELECT
mIDE_SELECT 	equ	0xff - 1<<bIDE_SELECT


; -------------------------------------------
; 		MEMORY LAYOUT
; -------------------------------------------

					; $0000 - $007F	  RST vectors, interrupt handler, eeprom helper 
					; $0080 - $00FF	  global variables
user_program	equ	$100		; $0100 - $1FFF	  user program
					;                 
dflt_stacks	equ	$2000		; $2000 - $20FF	  VM RStack (via IX)
					; $2100 - $25FF	  VM VStack (machine stack)
dflt_heap	equ	$2600		; $2600 - 	  dynamically allocated memory
					;       - $7EFF	  handles for allocated memory
;sio_buffers	equ	$7F00		; $7F00 - $7FFF	  SIO buffers
ram_end		equ	$8000;sio_buffers



; -------------------------------------------
;		CONSTANTS
; -------------------------------------------

; K1-bus:
;
k1_wr_sel	equ	$00		; A765=000	select board
k1_wr_irpt	equ	$40		; A765=010	write interrupt enable mask to boards
k1_wr_hi	equ	$80		; A765=100	write byte to lo-to-hi data latch
k1_wr_data	equ	$c0		; A765=110	write data to selected board; A0-A5 select register

#if option_a5 
k1_rd_irpt	equ	$20		; A765=001	read interrupt state bits (0=active)
k1_rd_i2c	equ	$60		; A765=011	read or write data to i2c bus
k1_rd_data	equ	$a0		; A765=101	read data from selected board; A0-A5 select register
k1_rd_hi	equ	$e0		; A765=111	read data from hi-to-low data latch
#else
k1_rd_irpt	equ	$00		; A76=00	read interrupt state bits (0=active)
k1_rd_i2c	equ	$40		; A76=01	read or write data to i2c bus
k1_rd_data	equ	$80		; A76=10	read data from selected board; A0-A5 select register
k1_rd_hi	equ	$c0		; A76=11	read data from hi-to-low data latch
#endif

i2c_d0_c0	equ	k1_rd_i2c	; i2c data=0 clock=0	$08 = A3 = data
i2c_d0_c1	equ	k1_rd_i2c | $10	; i2c data=0 clock=1	$10 = A4 = clock
i2c_d1_c0	equ	k1_rd_i2c | $08	; i2c data=1 clock=0
i2c_d1_c1	equ	k1_rd_i2c | $18	; i2c data=1 clock=1	= idle bus
i2c_idle	equ	i2c_d1_c1
yellow_led	equ	i2c_d1_c0	; yellow led == i2c clock=0  NOTE: avoid setting red without yellow: 
red_led		equ	i2c_d0_c1	; red led    == i2c data=0	   this is the i2c start condition!
both_leds	equ	i2c_d0_c0	; yellow_led + red_led
no_led		equ	i2c_d1_c1

k1_i2c_din	equ	$80		; mask for D7 = i2c data_in bit


; -------------------------------------------
; 		OTHER CONSTANTS
; -------------------------------------------



; ===========================================
; 		MEMORY LAYOUT
; ===========================================

; RAM:
#data 	DATA, page0end

; ROM:
#code 	GSINIT, $8000	; boot code; rom starts at $8000
#code 	PAGE0		; initializer data for RST etc. and memory management
	.phase $0000	; data will be copied to ram at $0000 => use virtual origin for convenience
#code	VM_CORE
#code 	VM_BYTE
#code	VM_INT
#code	VM_LONG
#code	VM_FLOAT
#code	VM_STRINGS
#code	VM_MEMORY
#code	TEST_CODE	; Test suite
#code	HW_DRIVER   	; e.g. SIO handler
#code 	CODE		; mostly Vcc code
#code	CONST		; const data
#code	ROMEND		; for easy calculation



; ===========================================
;	now: adding actual code and data
;	switching between segments as needed
; ===========================================

#code GSINIT
; after power-on the rom is also visible at $0000 so that the Z80 can start here
; we jump immediately to the standard mapping address so we can switch off init mode
	
reset::	jp	8003$
8003$:	di				; disable interrupts (if we jump here deliberately from the running code)
	im	1			; init interrupt mode
	
; reset k1 bus:
	ld	a, 0xff			; note: the first IORQ also resets the INIT flip flop => ram becomes accessible
	out	k1_wr_sel, a		; => deselect all boards (security only)
	out	k1_wr_irpt, a		; => disable interrupts on k1-bus boards
	in	a, i2c_idle 		; => set i2c bus to idle state, note: switches off the debug LEDs 

; test for SDP write protected eeprom in ram socket:
	ld	h,0			; hl -> ram
	inc	(hl)			; write to ram
	ld	a,(hl)
	xor	(hl)			; 0x00 expected
	cp	0x40			; 0x40 -> D6 toggled -> eeprom
	jp	z,unlock_eeprom_in_ram_socket	

; init sio1 UART early,
; so that the user has enough time to press "F" for firmware download:
	ld	hl,sio1_init_data
	ld	b, sio1_init_data_count
7$:	ld	c,(hl++)
	ld	a,(hl++)
	out	(c),a
	djnz	7$
	in	a,(k1_rd_data+STCR)    	; start C/T 
	in	a,(k1_rd_data+GPR)	; check test value in GPR
	cp	123
	ld	a,ERROR_SIO_NOT_FOUND
	jp	nz,panic

; test & clear ram to $00		
#include "asm/ramtest.s"		; -> hl = ramtop = 0x8000
	ld	sp,dflt_heap
	ld	ix,dflt_stacks + 0x80

; copy rst vectors, interrupt vectors and other initialized global variables:
	ld	hl, PAGE0
	ld	de, 0
	ld	bc, page0end - 0
	ldir					

; SDP write-protect eeprom:
	call	lock_eeprom
	jr	c,11$			; CY -> error -> not an eeprom
	ld	hl,sysflags
	inc	(hl)			; OK -> set bEEPROM in sysflags

11$:
; download firmware if uppercase "F" was received on sioA:
	;ld	a,mSIO_SELECT		; sio2 still selected from above
	;out	(k1_wr_sel),a
	call	sioA_test_char
	jr	z,12$			; no key pressed
	cp	a,"F"
	jr	nz,12$			; no "F"
	call	sioA_send_msg
	db	13,10,"download firmware ",0
	call	asm_download_firmware	; may return with interrupts enabled if user bails out	
	di				; no real risk: k1-bus irpt enable mask not yet written

;11$:
; print first boot message:
12$:	call	sioA_send_msg
	db	13,10,"ROM "
romdate:: dm	__date__,0

	call	sioA_send_msg
	db	" "
romtime:: dm	__time__,0
	
; initialization of globals and statics:
	;jp	$			



; ===========================================
; 	    INCLUDE EVERYTHING
; ===========================================


#include "VM/core-macros.s"
#include "asm/page0.s"
#include "asm/abort.s"
#include "asm/eeprom.s"		; has page0 code
#include "asm/sio.s"
#include "asm/i2c.s"
#include "asm/ide.s"
#include "VM/core.s"

;#include "asm/test_core.s"
;#include "asm/test16.s"
;#include "asm/test8.s"


#code GSINIT
	call	sioA_send_msg
	db	13,10,"booting..",0
	
	; now switch to virtual mode for Vcc initializer code:
	virtual


#cpath 	 "/pub/Develop/Projects/vicci/build/debug/vcc"	
#cflags  -cpu=vss -DDEBUG -o5 -v -c
#include "vcc/FD.cc"
#include "vcc/BlockDevice.cc"
#include "vcc/sio.cc"
#include "vcc/i2c.cc"
#include "vcc/ide.cc"
#include "vcc/File.cc"
#include "vcc/Dir.cc"
#include "vcc/test_core.cc"
#include "vcc/test_16.cc"
#include "vcc/test_8.cc"
#include "vcc/test_32.cc"
#include "vcc/test_str.cc"
#include "vcc/main.cc"			; GSINIT should not return

#code GSINIT
	dw	PANIC
	db	ERROR_END_OF_CODE	; blink code


; ________________________________________________________
#code PAGE0
page0end 	= $
page0_size 	= $ - $0000

#code ROMEND
romend 		= $
rom_size	= $ - $8000

#code 	CODE
code_size	= $ - CODE

#code	CONST
const_size	= $ - CONST

#code 	GSINIT
gsinit_size	= $ - GSINIT

#code	HW_DRIVER
hw_driver_size	= $ - HW_DRIVER

#code	VM_CORE
vm_core_size 	= $ - VM_CORE

#code 	VM_BYTE
vm_byte_size	= $ - VM_BYTE

#code	VM_INT
vm_int_size	= $ - VM_INT

#code	VM_LONG
vm_long_size 	= $ - VM_LONG

#code	VM_FLOAT
vm_float_size	= $ - VM_FLOAT

#code	VM_STRINGS
vm_strings_size	= $ - VM_STRINGS

#code	VM_MEMORY
vm_memory_size	= $ - VM_MEMORY

vm_size		= vm_core_size + vm_byte_size + vm_int_size + vm_long_size + vm_float_size + vm_strings_size + vm_memory_size	

#code	TEST_CODE
test_code_size	= $ - TEST_CODE
